@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 from __future__ import print_function
 
 import os
@@ -11,15 +10,11 @@ try:
 except:
 	pass
 
-
-
-
 def run(cmd,verbose=True):
 	print
 	print("------------------------------------------------------------------------------------------------")
 	print("running ",cmd)
 
-
 	try:
 		out=subprocess.getstatusoutput(cmd)
 	except:
@@ -31,7 +26,6 @@ def run(cmd,verbose=True):
 	print("------------------------------------------------------------------------------------------------")
 	print
 
-
 def get_resolution_array(rmf):
 
 	ind_matrix=-1
@@ -39,7 +33,6 @@ def get_resolution_array(rmf):
 
 	pf_buffer=pf.open(rmf)
 
-
 	for i in range(1,len(pf_buffer)):
 		extname=pf_buffer[i].header['EXTNAME']
 		print(extname)
@@ -85,7 +78,6 @@ def get_resolution_array(rmf):
 
 		## These are ill-built matrixes, typically Suzaku
 
-
 		#print f_chan[i][0],n_chan[i][0]
 		#print type(f_chan[i]).__name__
 		#print isinstance(f_chan[i], (list, tuple, np.ndarray))
@@ -99,7 +91,6 @@ def get_resolution_array(rmf):
 				emin_local=np.concatenate([emin_local,emin[f_chan[i][j]-chan_offset:f_chan[i][j]-chan_offset+n_chan[i][j]]])
 				emax_local=np.concatenate([emax_local,emax[f_chan[i][j]-chan_offset:f_chan[i][j]-chan_offset+n_chan[i][j]]])
 		else:
-
 			emin_local=emin[f_chan[i]-chan_offset:f_chan[i]-chan_offset+n_chan[i]]
 			emax_local=emax[f_chan[i]-chan_offset:f_chan[i]-chan_offset+n_chan[i]]
 			#print n_grp[i],f_chan[i],n_chan[i]
@@ -169,27 +160,11 @@ help+='XMM EpicPN eref=6 res=0.175 en_dep = -0.5\n'
 help+='XMM EpicMOS eref=6 res=0.155 en_dep = -0.5\n'
 help+='\n\nFor bugs and requests email to carlo.ferrigno AT unige.ch\n'
 help+='The script can be downloaded from the page http://www.isdc.unige.ch/~ferrigno/index.php/optimal-binning-script\n'
-#
-# parser = argparse.ArgumentParser(description='Optimal grouping for the spectrum',formatter_class=argparse.ArgumentDefaultsHelpFormatter)
-# parser.add_argument('Spectrum', metavar='fname', type=str, nargs=1,
-# 					help='Spectrum PHA1 file')
-# parser.add_argument("-b", "--backname", help="Background file", type=str, default='none')
-# parser.add_argument("-r", "--respname", help="RMF or RSP file (response matrix)", type=str, default='none')
-# parser.add_argument("-a", "--ancrname", help="ARF file (effective area)", type=str, default='none')
-# parser.add_argument("-e", "--mine", help="Minimum energy (keV), default is minimum of response", type=float, default=-10)
-# parser.add_argument("-E", "--maxe", help="Maximum energy (keV), default is maximum of response", type=float, default=1e10)
-# parser.add_argument("-m", "--mincounts", help="Minimum number of counts per bin (for Xspec >0, for other instruments also 0)", type=float, default=-1e10)
-# parser.add_argument("--eref", help="reference energy for custom resolution (keV)", type=float, default=-1)
-# parser.add_argument("--res", help="resolution at the reference energy for custom input (keV)", type=float, default=-1)
-# parser.add_argument("--en_dep", help="energy dependence of resolution (power-law index) for custom input (keV)", type=float, default=-1)
-
 
 def execute_binning(spectrum, bkg, rmf, arf, mine=-10, maxe=1e10, min_counts=0, eref=-1, res=-1, en_dep=-1):
-
     if mine >= maxe:
         raise RuntimeError("MinE > = MaxE (%f,%f)"%(mine,maxe))
 
-
     if (eref <=0 and res >= 0) or (eref >=0 and res <=0):
         raise RuntimeError('Optional arguments must be provided with positive reference energy and resolution\n\n'+help)
 
@@ -272,9 +247,9 @@ def execute_binning(spectrum, bkg, rmf, arf, mine=-10, maxe=1e10, min_counts=0,
 
 
     if mine == -10 or mine == -1e10:
-        mine=energy.min()
+        mine = energy.min()
     if maxe == 1e10 or maxe == -10:
-        maxe=energy.max()
+        maxe = energy.max()
 
     if mine < energy.min():
         mine = energy.min()
@@ -285,54 +260,53 @@ def execute_binning(spectrum, bkg, rmf, arf, mine=-10, maxe=1e10, min_counts=0,
     # Print input values
     #
 
-    parameter_string="\nWe will use the following input parameters:\n"
-    parameter_string+="spectrum   = %s\n"%(spectrum)
+    parameter_string = "\nWe will use the following input parameters:\n"
+    parameter_string += "spectrum   = %s\n"%(spectrum)
     if(eref >0 and res >0):
-        parameter_string+="res        = %g keV\n"%(res)
-        parameter_string+="E_ref      = %g keV\n"%(eref)
-        parameter_string+="E_dep      = %g\n"%(en_dep)
+        parameter_string += "res        = %g keV\n"%(res)
+        parameter_string += "E_ref      = %g keV\n"%(eref)
+        parameter_string += "E_dep      = %g\n"%(en_dep)
     else:
-        parameter_string+="RMF        = %s\n"%(rmf)
+        parameter_string += "RMF        = %s\n"%(rmf)
 
-    parameter_string+="ARF        = %s\n"%(arf)
-    parameter_string+="background = %s\n"%(bkg)
-    parameter_string+="min_counts = %g\n"%(min_counts)
-    parameter_string+="Emin       = %g\n"%(mine)
-    parameter_string+="Emax       = %g\n"%(maxe)
-    parameter_string+="\n"
+    parameter_string += "ARF        = %s\n"%(arf)
+    parameter_string += "background = %s\n"%(bkg)
+    parameter_string += "min_counts = %g\n"%(min_counts)
+    parameter_string += "Emin       = %g\n"%(mine)
+    parameter_string += "Emax       = %g\n"%(maxe)
+    parameter_string += "\n"
 
     print(parameter_string)
-
-
-
     #
     # Read counts from the spectral files
     #
 
     #bkg_channel=src_channel
-    bkg_counts=np.zeros(len(src_counts))
-    bkg_exposure=1
-    bkg_backscal=1
+    bkg_counts = np.zeros(len(src_counts))
+    bkg_exposure = 1
+    bkg_backscal = 1
 
     #for lower case test
-    bkg_test=bkg
+    bkg_test = bkg
 
     print(bkg_test)
 
     if(bkg_test.lower() != 'none'):
 
-        buffer = pf.open(bkg)
+        buffer = pf.open(bkg, mode='update')
         try:
             bkg_exposure = float(buffer[1].header['EXPOSURE'])
         except:
             print("Not found background exposure, defaulting to the source exposure %g s"%src_exposure)
-            bkg_exposure=src_exposure
+            bkg_exposure = src_exposure
+            buffer[1].header['EXPOSURE'] = src_exposure
 
         try:
             bkg_backscal = float(buffer[1].header['BACKSCAL'])
         except (KeyError):
             print("No BACKSCAL keyword in %s defaulting to 1" % (bkg))
-            bkg_backscal = 1
+            bkg_backscal = src_backscal
+            buffer[1].header['BACKSCAL'] = src_backscal
 
         bkg_channel = buffer[1].data['CHANNEL']
 
@@ -349,7 +323,7 @@ def execute_binning(spectrum, bkg, rmf, arf, mine=-10, maxe=1e10, min_counts=0,
                 print("ERROR: Impossible to read COUNTS or RATE from background")
                 os.exit(1)
 
-
+        buffer.flush()
         buffer.close()
 
     #print "SRC", src_exposure, src_backscal
@@ -361,8 +335,12 @@ def execute_binning(spectrum, bkg, rmf, arf, mine=-10, maxe=1e10, min_counts=0,
     #if res <=0:
     #	res=np.mean(res_array)
 
-    r=(maxe-mine)/res
-    if r>1e4:
+    if bkg_backscal == 0:
+        print("BAckground backscale is zero, we set it the source backscale")
+        bkg_backscal = src_backscal
+
+    r = (maxe-mine)/res
+    if r > 1e4:
         print('Warning: the resolution is not accurate')
 
     net_counts=src_counts - bkg_counts * src_backscal / bkg_backscal
